################################################################################
##                      Raspberry Pi Pico-IoT-Projekte
##
##                        Testinstallation SQLite
## 
## IoT-Projekt 4: POC "SQLite"
##
## Dieses Skript beinhaltet eine Skript-Schnipsel-Sammlung zum Umgang mit
## einer SQLite-Datenbank. Die Schnipsel stellen nur Anregung zur späteren
## SQLite-Anwendung auf einem Raspberry Pi 4B dar.
## 
## Siehe auch: https://www.sqlite.org
##
## Ad-Oculos-Projekt, https://www.faes.de/ad-oculos/
## Günter Faes
## Version 0.1.2, 17.01.2024
## R-Version: 4.3.2 (2023-06-16)
##
################################################################################

## Installation SQLite-DB:
# install.packages("DBI")
# install.packages("RSQLite")

## Laden der Pakete:
library(RSQLite)
library(lubridate) # Zur Zeitformatierung

# Erstellen einer permanenten DB:
DBVerbindung <- dbConnect(RSQLite::SQLite(),
                          dbname = "Datenbank/TestDB.db",
                          SQLITE_RW)
############# Beispiel Iris-Daten ##########################
View(iris) # Iris-Datensatz

# Tabelle anlegen:
dbWriteTable(DBVerbindung, "iris_tabelle", iris)
# Tabelle prüfen:
Tabellen <- dbListTables(DBVerbindung); Tabellen
# Tabellenfelder (Iris-Merkmale) prüfen:
TFelder <- dbListFields(DBVerbindung, "iris_tabelle"); TFelder

## Ein paar Testabfragen
# auslesen der gesamten Tabelle:
Abfrage <- dbReadTable(DBVerbindung, "iris_tabelle")
View(Abfrage)

# Ein wenig konkreter: Petal.Length >= 1.7:
Abfrage <- dbGetQuery(DBVerbindung, 'SELECT * FROM iris_tabelle WHERE "Petal.Length" >= 1.7')
View(Abfrage)

################### Beispiel Weindaten #######################

## Beispieldaten als CSV-Datei laden:
Weindaten <- read.csv2("Alle_Weindaten.csv") # Siehe Abspann
# Weindaten Citation Request
View(Weindaten)

# Tabelle anlegen:
dbWriteTable(DBVerbindung, "Wein_Tabelle", Weindaten)
# Tabelle prüfen:
Tabellen <- dbListTables(DBVerbindung); Tabellen
# Tabellenfelder prüfen:
TFelder <- dbListFields(DBVerbindung, "Wein_tabelle"); TFelder

# Abfrage bezüglich der Qualitätsnote:
Abfrage <- dbGetQuery(DBVerbindung, 'SELECT * FROM Wein_tabelle WHERE Quality >= 7')
View(Abfrage)


##################### DB schließen ###########################

# DB schließen:
dbDisconnect(DBVerbindung)



#### Existierende DB verknüpfen und neuen Eintrag anhängen ####
DBVerbindung <- dbConnect(RSQLite::SQLite(),
                          dbname = "Datenbank/TestDB.db",
                          SQLITE_RW)

# Tabelle prüfen:
Tabellen <- dbListTables(DBVerbindung); Tabellen

# Testabfrage:
Abfrage <- dbGetQuery(DBVerbindung, 'SELECT * FROM iris_tabelle WHERE "Petal.Length" >= 1.7')
View(Abfrage)

# Einen neue Iris-Daten generieren:
IrisTemp <- iris[1,]
IrisTemp$Petal.Length <- 10.2  # Einfach mal einen auffälligen Wert zuweisen

# Iris-Daten der iris_tabelle anfügen:
dbWriteTable(DBVerbindung, "iris_tabelle", IrisTemp, append = TRUE)

# Testabfrage, wurde die neue Beobachtung angehängt?
Abfrage <- dbGetQuery(DBVerbindung, 'SELECT * FROM iris_tabelle WHERE "Petal.Length" >= 5')
View(Abfrage)

######## Tabelle neu anlegen und mit Daten füllen #########
# Da dieses Skript ein PoC zum 4. IoT-Projekt ist, wird eine
# dazu passende Tabelle angelegt.

# Pico-Daten-Beispiel:
Datum <- dmy_hms("11.12.2023 20:22:13")  # lubridate
SensorIndex <- 10495
SensorName <- "Tank A"
Zyklus <- 5
Messtemperatur <- 22.9375
ESW <- 1
WarngrenzeUnten <- 15
WarngrenzeOben <- 30
EreignisFlag <- "Nein"

PicoDaten <- data.frame(Datum,
                        SensorIndex,
                        SensorName,
                        Zyklus,
                        Messtemperatur,
                        ESW,
                        WarngrenzeUnten,
                        WarngrenzeOben,
                        EreignisFlag)

# Tabelle anlgegen:
dbWriteTable(DBVerbindung, "PicoDatenTabelle", PicoDaten, append = TRUE)


# Angelegte Tabellen prüfen und Testabfrage:
Tabellen <- dbListTables(DBVerbindung); Tabellen
Abfrage <- dbGetQuery(DBVerbindung, 'SELECT * FROM PicoDatenTabelle')
View(Abfrage)

##################### Tabelle entfernen ###########################
# Falls nötig, kann eine Tabelle auch aus der DB entfernt werden.
# Beispielhaft werden die Tabelle "iris_tabelle" und "PicoDatenTabelle"
# entfernt:

dbRemoveTable(DBVerbindung, "iris_tabelle")

# Prüfen, ob es geklappt hat:
Tabellen <- dbListTables(DBVerbindung); Tabellen

# Entfernen über SQL-Anweisung:
dbExecute(DBVerbindung, "DROP TABLE PicoDatenTabelle")

# Prüfen, ob es geklappt hat:
Tabellen <- dbListTables(DBVerbindung); Tabellen

# DB schließen:
dbDisconnect(DBVerbindung)



#### Skript-Ende #####
